@@ -26,12 +26,12 @@ module Ide.Types
 , ConfigDescriptor(..), defaultConfigDescriptor, configForPlugin
 , CustomConfig(..), mkCustomConfig
 , FallbackCodeActionParams(..)
-, FormattingType(..), FormattingMethod, FormattingHandler, mkFormattingHandlers
+, FormattingType(..), FormattingMethod, FormattingHandler
 , HasTracing(..)
 , PluginCommand(..), CommandId(..), CommandFunction, mkLspCommand, mkLspCmdId
 , PluginId(..)
 , PluginHandler(..), mkPluginHandler
-, HandlerM, runHandlerM, pluginGetClientCapabilities, pluginGetVirtualFile, pluginGetVersionedTextDoc, pluginSendNotification, pluginSendRequest, pluginWithIndefiniteProgress
+, HandlerM, runHandlerM, pluginGetClientCapabilities, pluginSendNotification, pluginSendRequest, pluginWithIndefiniteProgress
 , PluginHandlers(..)
 , PluginMethod(..)
 , PluginMethodHandler
@@ -64,7 +64,6 @@ import           Control.Lens                  (_Just, view, (.~), (?~), (^.),
 import           Control.Monad                 (void)
 import           Control.Monad.Error.Class     (MonadError (throwError))
 import           Control.Monad.IO.Class        (MonadIO)
-import           Control.Monad.Trans.Class     (MonadTrans (lift))
 import           Control.Monad.Trans.Except    (ExceptT, runExceptT)
 import           Data.Aeson                    hiding (Null, defaultOptions)
 import qualified Data.Aeson.Types              as A
@@ -260,9 +259,11 @@ data PluginConfig =
       , plcCallHierarchyOn  :: !Bool
       , plcCodeActionsOn    :: !Bool
       , plcCodeLensOn       :: !Bool
+      , plcInlayHintsOn     :: !Bool
       , plcDiagnosticsOn    :: !Bool
       , plcHoverOn          :: !Bool
       , plcSymbolsOn        :: !Bool
+      , plcSignatureHelpOn  :: !Bool
       , plcCompletionOn     :: !Bool
       , plcRenameOn         :: !Bool
       , plcSelectionRangeOn :: !Bool
@@ -277,9 +278,11 @@ instance Default PluginConfig where
       , plcCallHierarchyOn  = True
       , plcCodeActionsOn    = True
       , plcCodeLensOn       = True
+      , plcInlayHintsOn     = True
       , plcDiagnosticsOn    = True
       , plcHoverOn          = True
       , plcSymbolsOn        = True
+      , plcSignatureHelpOn  = True
       , plcCompletionOn     = True
       , plcRenameOn         = True
       , plcSelectionRangeOn = True
@@ -289,15 +292,17 @@ instance Default PluginConfig where
       }
 
 instance ToJSON PluginConfig where
-    toJSON (PluginConfig g ch ca cl d h s c rn sr fr st cfg) = r
+    toJSON (PluginConfig g ch ca ih cl d h s sh c rn sr fr st cfg) = r
       where
         r = object [ "globalOn"         .= g
                    , "callHierarchyOn"  .= ch
                    , "codeActionsOn"    .= ca
                    , "codeLensOn"       .= cl
+                   , "inlayHintsOn"     .= ih
                    , "diagnosticsOn"    .= d
                    , "hoverOn"          .= h
                    , "symbolsOn"        .= s
+                   , "signatureHelpOn"  .= sh
                    , "completionOn"     .= c
                    , "renameOn"         .= rn
                    , "selectionRangeOn" .= sr
@@ -501,6 +506,9 @@ instance PluginMethod Request Method_TextDocumentDefinition where
 instance PluginMethod Request Method_TextDocumentTypeDefinition where
   handlesRequest _ msgParams pluginDesc _ = pluginSupportsFileType msgParams pluginDesc
 
+instance PluginMethod Request Method_TextDocumentImplementation where
+  handlesRequest _ msgParams pluginDesc _ = pluginSupportsFileType msgParams pluginDesc
+
 instance PluginMethod Request Method_TextDocumentDocumentHighlight where
   handlesRequest _ msgParams pluginDesc _ = pluginSupportsFileType msgParams pluginDesc
 
@@ -511,6 +519,12 @@ instance PluginMethod Request Method_WorkspaceSymbol where
   -- Unconditionally enabled, but should it really be?
   handlesRequest _ _ _ _ = HandlesRequest
 
+instance PluginMethod Request Method_TextDocumentInlayHint where
+  handlesRequest = pluginEnabledWithFeature plcInlayHintsOn
+
+instance PluginMethod Request Method_InlayHintResolve where
+  handlesRequest = pluginEnabledResolve plcInlayHintsOn
+
 instance PluginMethod Request Method_TextDocumentCodeLens where
   handlesRequest = pluginEnabledWithFeature plcCodeLensOn
 
@@ -530,6 +544,9 @@ instance PluginMethod Request Method_TextDocumentHover where
 instance PluginMethod Request Method_TextDocumentDocumentSymbol where
   handlesRequest = pluginEnabledWithFeature plcSymbolsOn
 
+instance PluginMethod Request Method_TextDocumentSignatureHelp where
+  handlesRequest = pluginEnabledWithFeature plcSignatureHelpOn
+
 instance PluginMethod Request Method_CompletionItemResolve where
   -- See Note [Resolve in PluginHandlers]
   handlesRequest = pluginEnabledResolve plcCompletionOn
@@ -688,6 +705,11 @@ instance PluginRequestMethod Method_TextDocumentTypeDefinition where
         | Just (Just True) <- caps ^? (L.textDocument . _Just . L.typeDefinition . _Just . L.linkSupport) = foldl' mergeDefinitions x xs
         | otherwise = downgradeLinks $ foldl' mergeDefinitions x xs
 
+instance PluginRequestMethod Method_TextDocumentImplementation where
+    combineResponses _ _ caps _ (x :| xs)
+        | Just (Just True) <- caps ^? (L.textDocument . _Just . L.implementation . _Just . L.linkSupport) = foldl' mergeDefinitions x xs
+        | otherwise = downgradeLinks $ foldl' mergeDefinitions x xs
+
 instance PluginRequestMethod Method_TextDocumentDocumentHighlight where
 
 instance PluginRequestMethod Method_TextDocumentReferences where
@@ -711,7 +733,7 @@ instance PluginRequestMethod Method_TextDocumentPrepareRename where
     combineResponses _ _ _ _ (x :| _) = x
 
 instance PluginRequestMethod Method_TextDocumentHover where
-  combineResponses _ _ _ _ (mapMaybe nullToMaybe . toList -> hs :: [Hover]) =
+  combineResponses _ _ _ _ (mapMaybe nullToMaybe . toList -> (hs :: [Hover])) =
     if null hs
         then InR Null
         else InL $ Hover (InL mcontent) r
@@ -748,6 +770,9 @@ instance PluginRequestMethod Method_TextDocumentDocumentSymbol where
             si = SymbolInformation name' (ds ^. L.kind) Nothing parent (ds ^. L.deprecated) loc
         in [si] <> children'
 
+instance PluginRequestMethod Method_TextDocumentSignatureHelp where
+  combineResponses _ _ _ _ (x :| _) = x
+
 instance PluginRequestMethod Method_CompletionItemResolve where
   -- A resolve request should only have one response.
   -- See Note [Resolve in PluginHandlers]
@@ -810,6 +835,9 @@ instance PluginRequestMethod Method_TextDocumentSemanticTokensFull where
 instance PluginRequestMethod Method_TextDocumentSemanticTokensFullDelta where
   combineResponses _ _ _ _ (x :| _) = x
 
+instance PluginRequestMethod Method_TextDocumentInlayHint where
+  combineResponses _ _ _ _ x = sconcat x
+
 takeLefts :: [a |? b] -> [a]
 takeLefts = mapMaybe (\x -> [res | (InL res) <- Just x])
 
@@ -899,29 +927,15 @@ instance GCompare IdeNotification where
 
 -- | Restricted version of 'LspM' specific to plugins.
 --
--- We plan to use this monad for running plugins instead of 'LspM', since there
--- are parts of the LSP server state which plugins should not access directly,
--- but instead only via the build system. Note that this restriction of the LSP
--- server state has not yet been implemented. See 'pluginGetVirtualFile'.
+-- We use this monad for running plugins instead of 'LspM', since there are
+-- parts of the LSP server state which plugins should not access directly, but
+-- instead only via the build system.
 newtype HandlerM config a = HandlerM { _runHandlerM :: LspM config a }
   deriving newtype (Applicative, Functor, Monad, MonadIO, MonadUnliftIO)
 
 runHandlerM :: HandlerM config a -> LspM config a
 runHandlerM = _runHandlerM
 
--- | Wrapper of 'getVirtualFile' for HandlerM
---
--- TODO: To be replaced by a lookup of the Shake build graph
-pluginGetVirtualFile :: NormalizedUri -> HandlerM config (Maybe VirtualFile)
-pluginGetVirtualFile uri = HandlerM $ getVirtualFile uri
-
--- | Version of 'getVersionedTextDoc' for HandlerM
---
--- TODO: Should use 'pluginGetVirtualFile' instead of wrapping 'getVersionedTextDoc'.
--- At the time of writing, 'getVersionedTextDoc' of the "lsp" package is implemented with 'getVirtualFile'.
-pluginGetVersionedTextDoc :: TextDocumentIdentifier -> HandlerM config VersionedTextDocumentIdentifier
-pluginGetVersionedTextDoc = HandlerM . getVersionedTextDoc
-
 -- | Wrapper of 'getClientCapabilities' for HandlerM
 pluginGetClientCapabilities :: HandlerM config ClientCapabilities
 pluginGetClientCapabilities = HandlerM getClientCapabilities
@@ -1183,31 +1197,8 @@ type FormattingHandler a
   -> FormattingOptions
   -> ExceptT PluginError (HandlerM Config) ([TextEdit] |? Null)
 
-mkFormattingHandlers :: forall a. FormattingHandler a -> PluginHandlers a
-mkFormattingHandlers f = mkPluginHandler SMethod_TextDocumentFormatting ( provider SMethod_TextDocumentFormatting)
-                      <> mkPluginHandler SMethod_TextDocumentRangeFormatting (provider SMethod_TextDocumentRangeFormatting)
-  where
-    provider :: forall m. FormattingMethod m => SMethod m -> PluginMethodHandler a m
-    provider m ide _pid params
-      | Just nfp <- uriToNormalizedFilePath $ toNormalizedUri uri = do
-        mf <- lift $ pluginGetVirtualFile $ toNormalizedUri uri
-        case mf of
-          Just vf -> do
-            let (typ, mtoken) = case m of
-                  SMethod_TextDocumentFormatting -> (FormatText, params ^. L.workDoneToken)
-                  SMethod_TextDocumentRangeFormatting -> (FormatRange (params ^. L.range), params ^. L.workDoneToken)
-                  _ -> Prelude.error "mkFormattingHandlers: impossible"
-            f ide mtoken typ (virtualFileText vf) nfp opts
-          Nothing -> throwError $ PluginInvalidParams $ T.pack $ "Formatter plugin: could not get file contents for " ++ show uri
-
-      | otherwise = throwError $ PluginInvalidParams $ T.pack $ "Formatter plugin: uriToFilePath failed for: " ++ show uri
-      where
-        uri = params ^. L.textDocument . L.uri
-        opts = params ^. L.options
-
 -- ---------------------------------------------------------------------
 
-
 data FallbackCodeActionParams =
   FallbackCodeActionParams
     { fallbackWorkspaceEdit :: Maybe WorkspaceEdit
